<template>
  <div class="mod-config">
    <basic-container>
      <el-row>
        <el-col :span="13">
          <ttable
            :table="table"
            @currentChange="handleCurrentChange"
            :uploadInfo="uploadInfo"
            :prelang="prelang"
            :options="options"
            :ajaxFun="ajaxFun"
            ref="outsourcingOrderTable"
          >
            <template #toolbar>
              <el-dropdown @command="handleCommand">
                <el-button>
                  状态变更<i class="el-icon-arrow-down el-icon--right"></i>
                </el-button>
                <el-dropdown-menu slot="dropdown">
                  <el-dropdown-item v-for="(item, index) in stateTagArr" :key="index" :command="item.command" :disabled="item.disabled">
                    {{ item.eventName }}
                  </el-dropdown-item>
                </el-dropdown-menu>
              </el-dropdown>
              <el-dropdown @command="handleCommandOa" style="margin-left: 10px">
                <el-button>
                  审核变更<i class="el-icon-arrow-down el-icon--right"></i>
                </el-button>
                <el-dropdown-menu slot="dropdown">
                  <template v-for="(item, index) in oaStateTagArr">
                    <el-dropdown-item :key="index" :command="item.command" v-if="item.isShow"
                      >{{ item.eventName }}
                    </el-dropdown-item>
                  </template>
                </el-dropdown-menu>
              </el-dropdown>
            </template>
          </ttable>
        </el-col>
        <el-col :span="11" style="padding-left: 10px">
          <ttable
            :table="joinTable"
            @handleSelectionChange="joinHandleSelectionChange"
            :uploadInfo="joinUploadInfo"
            :prelang="joinPrelang"
            :options="joinOptions"
            :ajaxFun="joinAjaxFun"
            :paramObj="paramObj"
            ref="joinStockTable"
          >
            <template #toolbar></template>
          </ttable>
        </el-col>
      </el-row>
      <!-- 弹窗, 新增 / 修改 -->
      <table-form
        v-if="addOrUpdateVisible"
        ref="addOrUpdate"
        @refreshDataList="getData"
      ></table-form>
      <outsourcing-order-stock-form
        v-if="addOrUpdateJoinVisible"
        ref="addOrUpdateJoin"
        @refreshDataList="getJoinData"
        @refreshOutsourcingOrder="refreshOutsourcingOrder"
      ></outsourcing-order-stock-form>
    </basic-container>
    <el-dialog
      title="提示"
      :visible.sync="showForeignDialog"
      width="30%">
      <span>这是一段信息</span>
      <span slot="footer" class="dialog-footer">
        <el-button @click="showForeignDialog = false">取 消</el-button>
        <el-button type="primary" @click="showForeignDialog = false">确 定</el-button>
      </span>
    </el-dialog>
  </div>
</template>

<script>
import {
  fetchListOutsourcingOrder,
  changeState,
  batchPutOutsourcingOrder,
  approvalOaByOutsourcingOrderId
} from '@/api/plan/outsourcingorder'
import {
  fetchList,
  delByIds,
  updateOrder
} from '@/api/plan/outsourcingorderstock'
import { mapGetters } from 'vuex'
import ttable from '@/views/common/ztt-table.vue'
import TableForm from './outsourcingorder-form'
import OutsourcingOrderStockForm from './outsourcingorderstock-form'
import { value2label, dateTime2date } from '@/util/formatUtil'
import { log } from '@chenfengyuan/vue-qrcode'

export default {
  data() {
    return {
      showForeignDialog: false,
      ajaxFun: fetchListOutsourcingOrder,
      addOrUpdateVisible: false,
      addOrUpdateJoinVisible: false,
      currentOutsourcingOrder: null,
      stateTagArr: [
        {
          value: '01planned',
          label: '已计划',
          command: 'PLAN',
          eventName: '已计划',
          disabled: false,
          permitArr: ['02issued', '03completed', '04canceled']
        },
        {
          value: '02issued',
          label: '已下达',
          command: 'COMMAND',
          eventName: '已下达',
          disabled: false,
          permitArr: ['03completed', '04canceled', '01planned']
        },
        {
          value: '03completed',
          label: '已完成',
          command: 'COMPLETE',
          eventName: '已完成',
          disabled: false,
          permitArr: ['01planned', '02issued', '04canceled']
        },
        {
          value: '04canceled',
          label: '已取消',
          command: 'CANCEL',
          eventName: '已取消',
          disabled: false,
          permitArr: ['01planned', '02issued', '03completed']
        }
      ],
      oaStateTagArr: [
        {
          value: '01cancel',
          label: '待审核',
          command: 'CANCEL',
          eventName: '撤销',
          isShow: true
        },
        {
          value: '02submit',
          label: '审核中',
          command: 'SUBMIT',
          eventName: '送审',
          isShow: false
        },
        {
          value: '03pass',
          label: '通过',
          command: 'PASS',
          eventName: '通过',
          isShow: true
        },
        {
          value: '04return',
          label: '退回',
          command: 'RETURN',
          eventName: '退回',
          isShow: true
        }
      ],
      multipleSelection: [],
      uploadInfo: {
        // 是否展示上传EXCEL以及对应的url
        isShow: false,
        url: ''
      },
      dateTimeFilters: {},
      prelang: 'operation',
      options: {
        height: 300, // 默认高度-为了表头固定
        stripe: true, // 是否为斑马纹 table
        highlightCurrentRow: false, // 是否要高亮当前行
        border: true, // 是否有纵向边框
        lazy: false, // 是否需要懒加载
        fit: true, // 列的宽度是否自撑开
        multiSelect: false, //
        seqNo: true,
        isRefresh: true, // 是否显示刷新按钮
        isShowHide: true, // 是否显示显影按钮H
        isSearch: true, // 高级查询按钮
        defaultOrderBy: { column: 'createTime', direction: 'desc' }
      },
      table: {
        total: 0,
        currentPage: 1,
        pageSize: 20,
        data: [],
        // 标题
        column: [
          {
            minWidth: '120',
            prop: 'outsourcingOrderNo',
            label: '委外订单号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text',
            render: { fun: this.addOrUpdateHandle }
          },
          {
            minWidth: '120',
            prop: 'partNo',
            label: '零件号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'partName',
            label: '零件名称',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'qtyRequired',
            label: '需求数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'qtyFinished',
            label: '完成数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            width: '120',
            prop: 'requiredDate',
            label: '需求日期',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange',
            formatter: this.formatDate
          },
          {
            minWidth: '140',
            prop: 'foreignState',
            label: '外协直发状态',
            sort: true,
            isTrue: true,
            formatter: this.formatForeignState,
            isSearch: true,
            searchInfoType: 'select',
            optList: () => {
              return this.foreignStateList
            }
          },
          {
            minWidth: '140',
            prop: 'state',
            label: '状态',
            sort: true,
            isTrue: true,
            formatter: this.formatState,
            isSearch: true,
            searchInfoType: 'select',
            optList: () => {
              return this.stateTagArr
            }
          },
          {
            minWidth: '140',
            prop: 'oaState',
            label: 'OA审核状态',
            sort: true,
            isTrue: true,
            formatter: this.formatOaState,
            isSearch: true,
            searchInfoType: 'select',
            optList: () => {
              return this.oaStateTagArr
            }
          },
          {
            minWidth: '200',
            prop: 'remark',
            label: '备注',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            width: '120',
            prop: 'updateTime',
            label: '更新时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange',
            formatter: this.formatDate
          },
          {
            minWidth: '100',
            prop: 'updateUser',
            label: '更新人',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            width: '120',
            prop: 'createTime',
            label: '创建时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange',
            formatter: this.formatDate
          },
          {
            minWidth: '100',
            prop: 'createUser',
            label: '创建人',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          }
        ],
        toolbar: [
          {
            text: '新增',
            type: 'primary',
            fun: this.addOrUpdateHandle
          },
          {
            text: '发起委外OA流程',
            type: 'primary',
            loading: false,
            fun: this.launchOaProcess
          },
          // {
          //   text: '新增外协直发申请单',
          //   type: 'primary',
          //   loading: false,
          //   fun: this.showForeign
          // },
        ],
        operator:[
          {
            text: '打印发料单',
            type: 'text',
            size: 'small',
            show: {
              val: [
                '03pass'
              ],
              key: 'oaState'
            }
          },
        ],
        operatorConfig: {
          fixed: 'right',
          label: '操作',
          width: 100,
          minWidth: 100
        }
      },
      foreignStateList: [
          {
            label: '是',
            value: 0
          },{
            label: '否',
            value: 1
          }
      ],
      paramObj: { outsourcingOrderId: 0 },
      joinAjaxFun: fetchList,
      joinMultipleSelection: [],
      joinUploadInfo: {
        // 是否展示上传EXCEL以及对应的url
        isShow: false,
        url: ''
      },
      joinPrelang: 'operation',
      joinOptions: {
        height: 300, // 默认高度-为了表头固定
        stripe: true, // 是否为斑马纹 table
        highlightCurrentRow: false, // 是否要高亮当前行
        border: true, // 是否有纵向边框
        lazy: false, // 是否需要懒加载
        fit: true, // 列的宽度是否自撑开
        multiSelect: true, //
        seqNo: true,
        isRefresh: true, // 是否显示刷新按钮
        isShowHide: true, // 是否显示显影按钮
        isSearch: false, // 高级查询按钮
        defaultOrderBy: { column: 'id', direction: 'asc' }
      },
      joinTable: {
        total: 0,
        currentPage: 1,
        pageSize: 20,
        data: [],
        // 标题
        column: [
          {
            minWidth: '110',
            prop: 'partNo',
            label: '零件编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text',
            render: { fun: this.addOrUpdateStockHandle }
          },
          {
            minWidth: '110',
            prop: 'partName',
            label: '零件名称',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '100',
            prop: 'locNo',
            label: '库位号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '100',
            prop: 'partBatchNo',
            label: '零件批号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '110',
            prop: 'receiveQuantity',
            label: '库存接收数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'number'
          },
          {
            minWidth: '100',
            prop: 'partUnit',
            label: '单位',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          }
        ],
        toolbar: [
          {
            text: '库存接收',
            type: 'primary',
            fun: this.addOrUpdateStockHandle,
            disabled: false
          },
          {
            text: '取消接收',
            type: 'primary',
            fun: this.cancelReceive,
            disabled: false
          }
        ],
        operator: null,
        operatorConfig: {
          fixed: 'right',
          label: '操作',
          width: 100,
          minWidth: 100
        }
      }
    }
  },
  components: {
    ttable,
    TableForm,
    OutsourcingOrderStockForm
  },
  computed: {
    ...mapGetters(['permissions'])
  },
  methods: {
    //展示新增外协直发模态框
    // showForeign(){
    //   let data = this.currentOutsourcingOrder
    //   if(data){
    //     if(data.foreignState == "1"){
    //       this.$message.warning("该订单无需外协直发申请单")
    //       return
    //     }
    //     if(data.foreignState == "0" && data.foreignId!= null){
    //       this.$message.warning("该订单已有外协直发申请单")
    //       return
    //     }
    //     //
    //     this.$message.success("新增")
    //   }else{
    //     this.$message.warning("请选择一条委外订单")
    //   }
    // },
    // 获取数据列表
    getData() {
      this.$refs.outsourcingOrderTable.refreshData()
    },
    // 新增 / 修改
    addOrUpdateHandle(row) {
      this.addOrUpdateVisible = true
      this.$nextTick(() => {
        this.$refs.addOrUpdate.init(row == null ? null : row.id,null,row == null ? null : row.mpsId)
      })
    },
    // 发起OA审核流程
    launchOaProcess() {
      if (this.currentOutsourcingOrder) {
        let foreignState = this.currentOutsourcingOrder.foreignState
        let foreignId = this.currentOutsourcingOrder.foreignId
        console.log(this.currentOutsourcingOrder);
        if(foreignState == 0 && foreignId==null){
          this.$message.warning("请先新增外协直发申请单")
          return
        }
        this.table.toolbar[1].loading = true
        approvalOaByOutsourcingOrderId(this.currentOutsourcingOrder.id)
          .then((data) => {
            this.$message.success('oa审批流程发起成功')
            this.getData()
            this.table.toolbar[1].loading = false
          })
          .catch(() => {
            this.table.toolbar[1].loading = false
          })
      } else {
        this.$message.warning('请选择委外订单')
      }
    },

    // 格式化年月日
    formatDate(row, column, cellValue) {
      return dateTime2date(cellValue)
    },
    // 状态格式化
    formatState(row, column, cellValue) {
      return value2label(cellValue, this.stateTagArr)
    },
    //外协直发状态格式化
    formatForeignState(row,column,cellValue){
      this.foreignStateList.forEach(obj=>{
        if(obj.value==cellValue){
          cellValue =  obj.label
        }
      })
      return cellValue
    },
    // OA状态格式化
    formatOaState(row, column, cellValue) {
      return value2label(cellValue, this.oaStateTagArr)
    },
    // OA审核状态改变
    handleCommandOa(event) {
      const oaStateTag = this.oaStateTagArr.find(
        (item) => item.command === event
      )
      const oaEventName = oaStateTag.eventName
      if (this.currentOutsourcingOrder) {
        batchPutOutsourcingOrder(
          new Array({
            id: this.currentOutsourcingOrder.id,
            oaState: oaStateTag.value
          }),
          event
        ).then((response) => {
          const data = response.data
          if (data.code === 0) {
            this.$message.success(oaEventName + '成功')
            this.getData()
          } else {
            this.$message.error(oaEventName + '失败')
          }
        })
      } else {
        this.$message.warning('请选择委外订单')
      }
    },
    // 状态改变
    handleCommand(event) {
      const stateTag = this.stateTagArr.find((item) => item.command === event)
      const eventName = stateTag.eventName
      if (this.currentOutsourcingOrder) {
        changeState([this.currentOutsourcingOrder.id], event).then(
          (response) => {
            const data = response.data
            if (data.code === 0) {
              this.$message.success(eventName + '修改成功')
              this.getData()
            } else {
              this.$message.error(eventName + '修改失败')
            }
          }
        )
      } else {
        this.$message.warning('请选择委外订单')
      }
    },
    handleCurrentChange(row) {
      if (!row) {
        this.currentOutsourcingOrder = null
        this.paramObj = { outsourcingOrderId: 0 }
      } else if (
        !this.currentOutsourcingOrder ||
        this.currentOutsourcingOrder.id !== row.id
      ) {
        this.currentOutsourcingOrder = row
        this.paramObj = { outsourcingOrderId: row.id }
      }
      this.$nextTick(() => {
        this.getJoinData()
      })

      // 根据状态，禁用表头按钮
      // 循环自定义按钮，判断每个按钮的permitArr是否完全包含选中状态，若完全包含，则按钮亮，否则按钮灰
      for (let i = 0; i < this.stateTagArr.length; i++) {
        if (this.stateTagArr[i].permitArr.includes(row.state)) {
          this.stateTagArr[i].disabled = false
        } else {
          this.stateTagArr[i].disabled = true
        }
      }
    },

    getJoinData() {
      this.$refs.joinStockTable.getDataList()
    },
    joinHandleSelectionChange(val) {
      this.joinMultipleSelection = val
    },
    // 库存接收
    addOrUpdateStockHandle(row) {
      if (this.currentOutsourcingOrder) {
        if(this.currentOutsourcingOrder.foreignState == "0"){
          this.$message.error('外协直发不接收库存')
          return
        }
        this.addOrUpdateJoinVisible = true
        this.$nextTick(() => {
          this.$refs.addOrUpdateJoin.init(
            row == null ? null : row.id,
            this.currentOutsourcingOrder
          )
        })
      } else {
        this.$message.error('请选择委外订单')
      }
    },
    // 取消接收
    cancelReceive() {
      if (this.joinMultipleSelection && this.joinMultipleSelection.length > 0) {
        const ids = []
        for (let i = 0; i < this.joinMultipleSelection.length; i++) {
          ids.push(this.joinMultipleSelection[i].id)
        }
        delByIds(ids).then((response) => {
          this.$refs.joinStockTable.getDataList()
          this.$message.success('取消预留成功')
          this.refreshOutsourcingOrder()
        })
      } else {
        this.$message.warning('请选择需要取消接收的库存')
      }
    },
    // 刷新委外订单的完成数量
    refreshOutsourcingOrder() {
      if (this.currentOutsourcingOrder && this.currentOutsourcingOrder.id) {
        updateOrder(this.currentOutsourcingOrder.id).then((response) => {
          this.$set(
            this.currentOutsourcingOrder,
            'qtyFinished',
            response.data.data
          )
        })
      }
    }
  }
}
</script>
